/**
 * @file Button.cpp
 * @author apio (cloudapio.eu)
 * @brief A clickable component that triggers an action when pressed.
 *
 * @copyright Copyright (c) 2023, the Luna authors.
 *
 */

#include <ui/Button.h>
#include <ui/Mouse.h>

namespace ui
{
    Button::Button(Window* window, Widget* parent) : ui::Widget(window, parent)
    {
    }

    void Button::recalculate_true_rects()
    {
        if (!m_child) return;
        m_child->rect() = m_rect;
        m_child->recalculate_true_rects();
    }

    void Button::recalculate_pseudo_rects()
    {
        if (m_child)
        {
            m_child->recalculate_pseudo_rects();
            m_pseudo_rect = m_child->pseudo_rect();
        }

        if (m_preferred_rect.has_value()) m_pseudo_rect = m_preferred_rect.value();
    }

    void Button::set_action(os::Action&& action)
    {
        m_action = move(action);
    }

    Result<EventResult> Button::handle_mouse_move(Point position)
    {
        m_hovered = true;
        return m_child->handle_mouse_move(position);
    }

    Result<EventResult> Button::handle_mouse_leave()
    {
        m_hovered = m_clicked = false;
        return m_child->handle_mouse_leave();
    }

    Result<EventResult> Button::handle_mouse_down(Point position, int buttons)
    {
        auto result = TRY(m_child->handle_mouse_down(position, buttons));
        if (result == EventResult::DidNotHandle)
        {
            if (!m_clicked && (buttons == ui::MouseButtons::LEFT))
            {
                m_clicked = true;
                m_action();
            }
        }
        return EventResult::DidHandle;
    }

    Result<EventResult> Button::handle_mouse_up(Point position, int buttons)
    {
        if (buttons & ui::MouseButtons::LEFT) m_clicked = false;
        return m_child->handle_mouse_up(position, buttons);
    }

    Result<EventResult> Button::handle_key_event(const ui::KeyEventRequest& request)
    {
        return m_child->handle_key_event(request);
    }

    Result<void> Button::draw(Canvas& canvas)
    {
        return m_child->draw(canvas);
    }
}
